//--------------------------------------
// Pine Tree generator
//--------------------------------------
// Copyright (C) 1993 Nicholas Centanni
//--------------------------------------
#ifndef __OB_TREE_H
#define __OB_TREE_H

#ifndef __NB_VER_H
#include "nb_ver.h"
#endif
#ifndef __ENVIRON_H
#include "ob_env.h"
#endif

#ifdef BENCHMARK
#define MAX_TREES       5
#else
#define MAX_TREES      10
#endif
#define LINE_TOT       28
#define VERT_TOT       56
#define get_random(lo, hi) ((long)random(hi - lo + 1) + lo)

// Tree primitives
struct tr_vertex
{
  l_point world;
  i_screen pixel[2];
};

struct tr_line
{
  char st;      // Pointer to starting vertex
  char en;      // Pointer to ending vertex
  int active;   // Display flag
};

class Tree
{
 private:
   int height;                    // Tree total height in feet

 public:
   Viewer *viewer;                // Connection to the 3-D viewer class
   Graphics *graphics;            // Connection to the graphics class
   tr_vertex vertices[VERT_TOT];  // Vertex array
   tr_line lines[LINE_TOT];       // Line array
   long *base;                    // Quick pointer to world.z of tree base
   int det_max;                   // Records tree detail state
   long tree_z;                   // Current z_dist of Tree base;
   int fading_out;
   int fading;

   Tree(Environ *env);
   ~Tree(void) {}
   int translate(void);
   void build(long x, long y, long z);
   void draw(void);
   void erase(void);
   void detail(int state);
};

class Forest
{
 private:
   Environ *env;
   Graphics *graphics;
   int max_trees;
   int visible[MAX_TREES];
   long x_lim;
   long z_lim;
   long z_margin;
   Tree **tree;

 public:
   int tree_tot;
   int active_trees;
   Forest(Environ *e, int tree_max, long x_limit, long z_limit);
   ~Forest(void);
   void translate(void);
   void create_trees(void);
   void delete_trees(void);
   void add_tree(void);
   void draw(void);
   void draw_back(void);
   void draw_front(void);
   void erase(void);
   void move(long distance);
};
#endif
